import scrapy
from scrapy.spiders import Spider,CrawlSpider, Rule
import socket
import re
import json
from urllib.request import urlopen
from urllib.parse import urlparse
from utility.countries import isEuropean
import validators

class GFontsSpider(Spider):
    name = "gfonts"
    #start_urls = self.getUrls()
    checked_domains = []
    eu_domains = []
    custom_settings = {'CLOSESPIDER_PAGECOUNT': 2000}

    def __init__(self):
        self.start_urls = self.getUrls()

    def parse(self, response):
        # check current url - german or eu (todo)
        # check if api.gfonts.com is found
        # @todo: check if google analytics is found
        parsed = urlparse(response.url)
        if isEuropean(response.url):
            print("URL EUROPEAN: " + response.url)
            if parsed.hostname not in self.eu_domains:
                self.eu_domains.append(parsed.hostname)
                self.logNewDomain(response.url)
                yield self.writeTrackers(response)
        else:
            print("NOT EUROPEAN: " + response.url)
            self.checked_domains.append(parsed.hostname)
        for link in self.parseOn(response):
            yield scrapy.Request(link, callback=self.parse)

    def findGFonts(self, response):
        for link in response.css('head link'):
            try:
                href = link.attrib['href']
                if 'fonts.googleapis.com' in href:
                    return True
            except:
                continue
        return False

    def findGTrackers(self, response):
        trackers = { 'ga' : 'www.google-analytics.com',
                'gt' : 'www.googletagmanager.com'}
        result = {'ga':False, 'gt':False}
        for script in response.css('script::text').getall():
            if script.find(trackers['ga']) > 0: result['ga'] = True
            if script.find(trackers['gt']) > 0: result['gt'] = True
        return result

    def findMetaPixel(self, response):
        for img in response.css('img'):
            try:
                if img.attrib['src'].find('www.facebook.com/tr?id=') > 0: return True
            except:
                continue
        return False

    def writeTrackers(self,response):
        gtrackers = self.findGTrackers(response)
        return {
            'domain': urlparse(response.url).netloc,
            'country': isEuropean(response.url),
            'gf': self.findGFonts(response),
            'ga': gtrackers['ga'],
            'gt': gtrackers['gt'],
            'mp': self.findMetaPixel(response)
                }

    def parseOn(self, response):
        links = response.css('a');
        print('FOUND: ' + str(len(links)) + ' LINKS')
        next_urls = []
        for link in links:
            try:
                url = link.attrib['href']
                found = urlparse(url)
                if validators.url(url) and bool(found.netloc):
                    current = urlparse(response.url)
                    if current.hostname != found.hostname and found.hostname not in self.checked_domains:
                        next_urls.append(url)
            except:
                continue
        print('FOLLOW: ' + str(len(next_urls)) + ' LINKS')
        return next_urls

    def getUrls(self):
        with open('sites.txt') as sites_file:
            sites = sites_file.readlines()
            return map(lambda s: 'https://' + s,sites)

    def logNewDomain(self, url):
        print('############################################')
        print('######   ' + url + '   #######')
        print('############################################')

    def getEuTlds(self):
        return { '.ad': 'AD', '.at': 'AT', '.be':'BE', '.ch':'CH', '.cz':'CZ', 
        '.de':'DE', '.dk':'DK', '.ee':'EE', '.es':'ES', '.eu':'EU', '.fi':'FI',
        '.fr':'FR', '.gr':'GR', '.hr':'HR', '.hu':'HU', '.ie':'IE', '.it':'IT',
        '.li':'LI', '.lt':'LT', '.lu':'LU', '.lv':'LV', '.nl':'NL', '.no':'NO',
        '.pl':'PL', '.pt':'PT', '.ro':'RO', '.se':'SE', '.si':'SI', '.sk':'SK',
        '.bg':'BG', '.cy':'CY', '.mt':'MT'}
